/*
 *  SPDX-FileCopyrightText: 2019 Marco Martin <mart@kde.org>
 *
 *   SPDX-License-Identifier: LGPL-2.0-or-later
 */

import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Layouts 1.1
import QtGraphicalEffects 1.0

import org.kde.plasma.plasmoid 2.0
import org.kde.plasma.core 2.0 as PlasmaCore
import org.kde.plasma.components 3.0 as PlasmaComponents
import org.kde.draganddrop 2.0 as DragDrop

import org.kde.plasma.private.mobilecontainmentlayoutmanager 1.0 as ContainmentLayoutManager 

import org.kde.plasma.private.mobileshell 1.0 as MobileShell
import org.kde.phone.homescreen.default 1.0 as HomeScreenLib

import ru.astralinux.plasma.private.kiosk 1.0

import "private" as Private

DragDrop.DropArea {
    id: dropArea
    
    required property var homeScreenState
    
    property alias launcherDelegate: launcherRepeater.delegate
    property alias launcherModel: launcherRepeater.model
    property alias launcherRepeater: launcherRepeater
    property alias itemsBoundingRect: appletsLayout.childrenRect

    property alias appletsLayout: appletsLayout

    property FavoriteStrip favoriteStrip
    property HomeScreenPages homeScreenPages

    property LauncherDragManager launcherDragManager: LauncherDragManager {
        id: launcherDragManager
        parent: {
            let candidate = dropArea;
            while (candidate.parent) {
                candidate = candidate.parent;
            }
            return candidate;
        }
        anchors.fill: parent
        z: 999999
        appletsLayout: dropArea.appletsLayout
        favoriteStrip: dropArea.favoriteStrip
    }

    property bool inAppletEditMode: false
    
    property var applicationListModel: HomeScreenLib.ApplicationListModel
    property var desktopModel: HomeScreenLib.DesktopModel

    Connections {
        target: plasmoid
        function onEditModeChanged() {
            appletsLayout.editMode = plasmoid.editMode
        }
    }

    onDragEnter: {
        appletsLayout.appletsLayoutInteracted();
        event.accept(event.proposedAction);
        launcherDragManager.active = true;
    }
    
    onDragMove: {
        let posInFavorites = favoriteStrip.mapFromItem(this, event.x, event.y);
        if (posInFavorites.y > 0) {
            if (desktopModel.favoriteCount >= desktopModel.maxFavoriteCount) {
                launcherDragManager.hideSpacer();
            } else {
                launcherDragManager.showSpacerAtPos(event.x, event.y, favoriteStrip);
            }
            appletsLayout.hidePlaceHolder();
        } else {
            if (event.mimeData.formats[0] === "text/x-plasma-phone-homescreen-launcher") {
                // Show space for an item
                appletsLayout.showPlaceHolderAt(
                    Qt.rect(event.x - appletsLayout.cellWidth / 2,
                    event.y - appletsLayout.cellHeight / 2,
                    appletsLayout.cellWidth,
                    appletsLayout.cellHeight)
                );
            } else {
                // Show space for an applet
                appletsLayout.showPlaceHolderAt(
                    Qt.rect(event.x - appletsLayout.defaultItemWidth / 2,
                    event.y - appletsLayout.defaultItemHeight / 2,
                    appletsLayout.defaultItemWidth,
                    appletsLayout.defaultItemHeight)
                );
            }
            launcherDragManager.hideSpacer();

            let scenePos = mapToItem(null, event.x, event.y);
            //SCROLL LEFT
            if (scenePos.x < PlasmaCore.Units.gridUnit) {
                homeScreenPages.scrollLeft();
            //SCROLL RIGHT
            } else if (scenePos.x > homeScreenPages.width - PlasmaCore.Units.gridUnit) {
                homeScreenPages.scrollRight();
            //DON't SCROLL
            } else {
                homeScreenPages.stopScroll();
            }
        }
    }

    onDragLeave: {
        appletsLayout.hidePlaceHolder();
        launcherDragManager.active = false;
    }

    preventStealing: true

    onDrop: {
        launcherDragManager.active = false;
        if (event.mimeData.formats[0] === "text/x-plasma-phone-homescreen-launcher") {
            let storageId = event.mimeData.getDataAsByteArray("text/x-plasma-phone-homescreen-launcher");

            let posInFavorites = favoriteStrip.flow.mapFromItem(this, event.x, event.y);
            if (posInFavorites.y > 0) {
                if (desktopModel.favoriteCount < desktopModel.maxFavoriteCount ) {
                    launcherDragManager.dropFavoriteStorageId(storageId, posInFavorites.x, posInFavorites.y);
                }

                return;
            }

            launcherDragManager.dropDesktopStorageId(storageId, appletsLayout.placeHolder.x,  appletsLayout.placeHolder.y);
            appletsLayout.hidePlaceHolder();
            return;
        } else {
            plasmoid.processMimeData(event.mimeData,
                        event.x - appletsLayout.placeHolder.width / 2, event.y - appletsLayout.placeHolder.height / 2);
            event.accept(event.proposedAction);
            appletsLayout.hidePlaceHolder();
        }
    }

    ContainmentLayoutManager.AppletsLayout {
        id: appletsLayout

        anchors {
            fill: parent
            bottomMargin: dropArea.favoriteStrip ? dropArea.favoriteStrip.height : 0
        }

        signal appletsLayoutInteracted

        TapHandler {
            target: homeScreenPages
            enabled: homeScreenState.currentView === HomeScreenState.PageView && homeScreenState.interactive
            onTapped: {
                //Hides icons close button
                appletsLayout.appletsLayoutInteracted();
                appletsLayout.editMode = false;
                appletsLayout.forceActiveFocus();
            }
            onLongPressed: {
                if (homeScreenState.currentSwipeState === HomeScreenState.DeterminingType) {
                    // only go into edit mode when not in a swipe
                    appletsLayout.editMode = true;
                }
            }
            onPressedChanged: appletsLayout.focus = true;
        }

        property real lostSpace: 2 * PlasmaCore.Units.iconSizes.smallMedium

        cellWidth: Math.floor(favoriteStrip.width / appletsLayout.columnsNumberPerScreen)
        cellHeight: Math.floor((height - lostSpace) / appletsLayout.rowsNumberPerScreen)

        configKey: "ItemsGeometries"
        containment: plasmoid
        editModeCondition: plasmoid.immutable
                ? ContainmentLayoutManager.AppletsLayout.Manual
                : ContainmentLayoutManager.AppletsLayout.AfterPressAndHold
        kioskMode: Kiosk.enable

        // Sets the containment in edit mode when we go in edit mode as well
        onEditModeChanged: plasmoid.editMode = editMode;

        minimumItemWidth: PlasmaCore.Units.iconSizes.huge
        minimumItemHeight: favoriteStrip.availableCellHeight

        defaultItemWidth: appletsLayout.cellWidth
        defaultItemHeight: appletsLayout.cellHeight

        acceptsAppletCallback: function(applet, x, y) {
            return true;
        }
        appletContainerComponent: MobileAppletContainer {
            homeScreenState: dropArea.homeScreenState
            launcherDragManager: dropArea.launcherDragManager

            onEditModeChanged: {
                inAppletEditMode = editMode;
            }
        }

        placeHolder: ContainmentLayoutManager.PlaceHolder {}
        //FIXME: move
        PlasmaComponents.Label {
            id: metrics
            text: "M\nM"
            visible: false
            font.pointSize: PlasmaCore.Theme.defaultFont.pointSize * 0.9
        }
        LauncherRepeater {
            id: launcherRepeater
            homeScreenState: dropArea.homeScreenState
            cellWidth: appletsLayout.cellWidth
            cellHeight: appletsLayout.cellHeight
            appletsLayout: appletsLayout
            favoriteStrip: dropArea.favoriteStrip
            onScrollLeftRequested: homeScreenPages.scrollLeft()
            onScrollRightRequested: homeScreenPages.scrollRight()
            onStopScrollRequested: homeScreenPages.stopScroll()
        }
    }
}

