/*
 *   SPDX-FileCopyrightText: 2019 Marco Martin <mart@kde.org>
 *
 *   SPDX-License-Identifier: LGPL-2.0-or-later
 */

import QtQuick 2.4

import org.kde.plasma.plasmoid 2.0
import org.kde.plasma.private.mobilecontainmentlayoutmanager 1.0 as ContainmentLayoutManager 
import org.kde.plasma.private.mobileshell 1.0 as MobileShell
import org.kde.phone.homescreen.default 1.0 as HomeScreenLib

Item {
    id: root

    property ContainmentLayoutManager.AppletsLayout appletsLayout
    property FavoriteStrip favoriteStrip
    property ContainmentLayoutManager.ItemContainer currentlyDraggedDelegate
    property bool active
    
    property var desktopModel: HomeScreenLib.DesktopModel

    readonly property Item spacer: Item {
        width: favoriteStrip.cellWidth
        height: favoriteStrip.cellHeight
    }

    function setTextVisibility(item, value) {
        if (item.hasOwnProperty("showText")) {
            item.showText = value;
        }
    }

    function resetItemRelativePosition(item) {
        item.row = -1;
        item.column = -1;
    }

    function startDrag(item) {
        showSpacer(item, 0, 0);
    }

    function dragItem(delegate, dragCenterX, dragCenterY) {
              // newPosition
        var newRow = 0;

        var newContainer = internal.containerForItem(delegate, dragCenterX, dragCenterY);
        if (!newContainer) {
            newContainer = appletsLayout;
        }

        showSpacer(delegate, dragCenterX, dragCenterY);
    }

    function dropItem(item, dragCenterX, dragCenterY) {
        internal.positionItem(item, dragCenterX, dragCenterY);
    }

    function dropFavoriteStorageId(storageId, dragCenterX, dragCenterY) {
        var container = favoriteStrip;
        var child = internal.nearestChildFromPos(dragCenterX, dragCenterY, container);

        internal.raiseContainer(container);

        if (HomeScreenLib.DesktopModel.favoriteCount == 0 || child.modelData === undefined) {
            HomeScreenLib.DesktopModel.addFavorite(storageId, 0, HomeScreenLib.ApplicationListModel.Favorites);
        } else if (dragCenterX < child.x + child.width / 2) {
            HomeScreenLib.DesktopModel.addFavorite(storageId, child.modelData.index, HomeScreenLib.ApplicationListModel.Favorites);
        } else {
            HomeScreenLib.DesktopModel.addFavorite(storageId, child.modelData.index+1, HomeScreenLib.ApplicationListModel.Favorites);
        }
        hideSpacer();
    }

    function dropDesktopStorageId(storageId, dragCenterX, dragCenterY) {
        let pos = HomeScreenLib.DesktopModel.count;
        HomeScreenLib.DesktopModel.addFavorite(storageId, pos, HomeScreenLib.ApplicationListModel.Desktop);

        let item = appletsLayout.childAt(0,0); // hack for item

        if (item.modelData || false) {
            HomeScreenLib.DesktopModel.setLocation(item.modelData.index, HomeScreenLib.ApplicationListModel.Desktop);
        }

        item.parent = appletsLayout;
        item.x = dragCenterX;
        item.y = dragCenterY;
        setTextVisibility(item, true);
        resetItemRelativePosition(item);
        appletsLayout.hidePlaceHolder();
        appletsLayout.positionItem(item);
    }

    function showSpacer(item, dragCenterX, dragCenterY) {
        var container = internal.containerForItem(item, dragCenterX, dragCenterY);

        internal.raiseContainer(container);

        appletsLayout.hidePlaceHolder();

        if (container == appletsLayout) {
            spacer.visible = false;
            spacer.parent = root;
            appletsLayout.releaseSpace(item);
            internal.putItemInDragSpace(item);
            let pos = appletsLayout.mapFromItem(item, 0, 0);
            appletsLayout.showPlaceHolderAt(Qt.rect(pos.x, pos.y, item.width, item.height));
            return;
        }

        var child = internal.nearestChild(item, dragCenterX, dragCenterY, container);

        if (!child) {
            spacer.visible = false;
            spacer.parent = container.flow
            spacer.visible = true;
            return;
        }

        spacer.visible = false;
        spacer.parent = container.flow

        var pos = container.flow.mapFromItem(item, dragCenterX, dragCenterY);

        if (pos.x < child.x + child.width / 2) {
            MobileShell.ShellUtil.stackItemBefore(spacer, child);
        } else {
            MobileShell.ShellUtil.stackItemAfter(spacer, child);
        }

        internal.putItemInDragSpace(item);

        spacer.visible = true;
    }

    function showSpacerAtPos(x, y, container) {
        var pos = container.flow.mapFromGlobal(x, y);
        internal.raiseContainer(container);

        appletsLayout.hidePlaceHolder();

        if (container == appletsLayout) {
            spacer.visible = false;
            spacer.parent = root;
            appletsLayout.showPlaceHolderAt(Qt.rect(pos.x, pos.y, appletsLayout.cellWidth, appletsLayout.cellHeight));
            return;
        }

        var child = internal.nearestChildFromPos(x, y, container);

        if (!child) {
            spacer.visible = false;
            spacer.parent = container.flow
            spacer.visible = true;
            return;
        }

        spacer.visible = false;
        spacer.parent = container.flow

        if (pos.x < child.x + child.width / 2) {
            MobileShell.ShellUtil.stackItemBefore(spacer, child);
        } else {
            MobileShell.ShellUtil.stackItemAfter(spacer, child);
        }

        spacer.visible = true;
    }

    function hideSpacer () {
        spacer.visible = false;
        spacer.parent = root;
    }

    // Those should never be accessed from outside
    QtObject {
        id: internal
        function raiseContainer(container) {
            container.z = 1;

            if (container == appletsLayout) {
                favoriteStrip.z = 0;
            } else if (container == favoriteStrip) {
                appletsLayout.z = 0;
            } else {
                appletsLayout.z = 0;
                favoriteStrip.z = 0;
            }
        }

        function containerForItem(item, dragCenterX, dragCenterY) {
            if (!item.modelData) {
                return appletsLayout;
            } else if (favoriteStrip.contains(Qt.point(0,favoriteStrip.frame.mapFromItem(item, dragCenterX, dragCenterY).y))
                       && ((item.modelData.applicationLocation == HomeScreenLib.DesktopModel.Favorites
                            && HomeScreenLib.DesktopModel.hasFavorite(item.modelData.applicationUniqueId))
                           || HomeScreenLib.DesktopModel.favoriteCount < HomeScreenLib.DesktopModel.maxFavoriteCount)) {
                return favoriteStrip;
            } else {
                return appletsLayout;
            }
        }

        function putItemInDragSpace(item) {
            var pos = root.mapFromItem(item, 0, 0);
            item.parent = root;

            item.x = pos.x;
            item.y = pos.y;
        }

        function putInContainerLayout(item, container) {
            var pos = container.flow.mapFromItem(item, 0, 0);

            if (container == appletsLayout) {
                item.parent = container;
                item.modelData.applicationLocation = HomeScreenLib.ApplicationListModel.Desktop;
            } else if(container == favoriteStrip) {
                item.parent = container;
                item.modelData.applicationLocation = HomeScreenLib.ApplicationListModel.Favorites;
            } else {
                item.parent = container.flow;
            }

            item.x = pos.x;
            item.y = pos.y;
        }

        function nearestChild(item, dragCenterX, dragCenterY, container) {
            var distance = Number.POSITIVE_INFINITY;
            var child;
            var pos = container.flow.mapFromItem(item, dragCenterX, dragCenterY);

            // Search Right
            for (var i = 0; i < item.width * 2; i += item.width/2) {
                var candidate = container.flow.childAt(
                    Math.min(container.flow.width, Math.max(0, pos.x + i)),
                    Math.min(container.flow.height-1, Math.max(0, pos.y)));

                if (candidate && i < distance) {
                    child = candidate;
                    break;
                }
            }

            // Search Left
            for (var i = 0; i < item.width * 2; i += item.width/2) {
                var candidate = container.flow.childAt(Math.min(container.flow.width, Math.max(0, pos.x - i)), Math.min(container.flow.height-1, Math.max(0, pos.y)));

                if (candidate && i < distance) {
                    child = candidate;
                    break;
                }
            }

            if (!child) {
                if (item.y < container.flow.height/2) {
                    child = container.flow.children[0];
                } else {
                    child = container.flow.children[container.flow.children.length - 1];
                }
            }

            return child;
        }


        function nearestChildFromPos(x, y, container) {
            var distance = Number.POSITIVE_INFINITY;
            var child;
            var pos = container.flow.mapFromGlobal(x, y);
    
            // Search Right
            for (var i = 0; i < appletsLayout.cellWidth * 2; i += appletsLayout.cellWidth/2) {
                var candidate = container.flow.childAt(
                    Math.min(container.flow.width, Math.max(0, pos.x + i)),
                    Math.min(container.flow.height-1, Math.max(0, pos.y)));

                if (candidate && i < distance) {
                    child = candidate;
                    break;
                }
            }

            // Search Left
            for (var i = 0; i < appletsLayout.cellWidth * 2; i += appletsLayout.cellWidth/2) {
                var candidate = container.flow.childAt(Math.min(container.flow.width, Math.max(0, pos.x - i)), Math.min(container.flow.height-1, Math.max(0, pos.y)));

                if (candidate && i < distance) {
                    child = candidate;
                    break;
                }
            }

            return child;
        }

        function stackAfterToArray(arr, val, orig) {
            var res = [];

            for (var value in Object.entries(arr)) {
                res.push(arr[value]);
                if(arr[value] == orig) {
                    res.push(val);
                }
            }

            return res;
        }

        function stackBeforeToArray(arr, val, orig) {
            var res = [];

            for (var value in Object.entries(arr)) {
                if(arr[value] == orig) {
                    res.push(val);
                }
                res.push(arr[value]);
            }

            return res;
        }

        function positionItem(item, dragCenterX, dragCenterY) {
            hideSpacer();
            var container = containerForItem(item, dragCenterX, dragCenterY);

            raiseContainer(container);

            if (container == favoriteStrip) {
                var child = nearestChild(item, dragCenterX, dragCenterY, container);

                setTextVisibility(item, false);

                let flowChildrens = [];
                if(child == undefined) {
                    HomeScreenLib.DesktopModel.addFavorite(item.modelData.applicationUniqueId, 0, HomeScreenLib.ApplicationListModel.Favorites);
                    putInContainerLayout(item, container);
                    flowChildrens = [item];
                } else if (favoriteStrip.flow.mapFromItem(item, 0, 0).x < child.x + child.width / 2) {
                    HomeScreenLib.DesktopModel.addFavorite(item.modelData.applicationUniqueId, child.modelData.index, HomeScreenLib.ApplicationListModel.Favorites);
                    flowChildrens = internal.stackBeforeToArray(favoriteStrip.flow.children, item, child);
                } else {
                    HomeScreenLib.DesktopModel.addFavorite(item.modelData.applicationUniqueId, child.modelData.index+1, HomeScreenLib.ApplicationListModel.Favorites);
                    flowChildrens = internal.stackAfterToArray(favoriteStrip.flow.children, item, child);
                }

                favoriteStrip.flow.children = flowChildrens;

                spacer.visible = false;
                spacer.parent = root;

                return;

            } else if (container == appletsLayout) {
                if (item.modelData) {
                    HomeScreenLib.DesktopModel.setLocation(item.modelData.index, HomeScreenLib.ApplicationListModel.Desktop);
                }
                let pos = appletsLayout.mapFromItem(item, 0, 0);
                item.parent = appletsLayout;
                item.x = pos.x;
                item.y = pos.y;
                setTextVisibility(item, true);
                resetItemRelativePosition(item);
                appletsLayout.hidePlaceHolder();
                appletsLayout.positionItem(item);

                return;
            }  else {
                setTextVisibility(item, true);
                HomeScreenLib.DesktopModel.setLocation(item.modelData.index, HomeScreenLib.ApplicationListModel.Grid);
            }

            putInContainerLayout(item, container);
            MobileShell.ShellUtil.stackItemBefore(item, spacer);
            spacer.visible = false;
            spacer.parent = root;
        }
    }
}


